@@ -53,6 +53,143 @@ const gchar *papersize = NULL;
 gboolean enable_scripts = TRUE;
 gboolean print_backgrounds = FALSE;
 
+#ifndef POPPLER_WITH_GDK
+#include <goo/gtypes.h>
+
+static void load_done(WebKitWebFrame *frame, WebKitLoadStatus status);
+
+static void
+copy_cairo_surface_to_pixbuf (cairo_surface_t *surface,
+			      GdkPixbuf       *pixbuf)
+{
+  int cairo_width, cairo_height, cairo_rowstride;
+  unsigned char *pixbuf_data, *dst, *cairo_data;
+  int pixbuf_rowstride, pixbuf_n_channels;
+  unsigned int *src;
+  int x, y;
+
+  cairo_width = cairo_image_surface_get_width (surface);
+  cairo_height = cairo_image_surface_get_height (surface);
+  cairo_rowstride = cairo_image_surface_get_stride (surface);
+  cairo_data = cairo_image_surface_get_data (surface);
+
+  pixbuf_data = gdk_pixbuf_get_pixels (pixbuf);
+  pixbuf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+  pixbuf_n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+
+  if (cairo_width > gdk_pixbuf_get_width (pixbuf))
+    cairo_width = gdk_pixbuf_get_width (pixbuf);
+  if (cairo_height > gdk_pixbuf_get_height (pixbuf))
+    cairo_height = gdk_pixbuf_get_height (pixbuf);
+  for (y = 0; y < cairo_height; y++)
+    {
+      src = (unsigned int *) (cairo_data + y * cairo_rowstride);
+      dst = pixbuf_data + y * pixbuf_rowstride;
+      for (x = 0; x < cairo_width; x++) 
+	{
+	  dst[0] = (*src >> 16) & 0xff;
+	  dst[1] = (*src >> 8) & 0xff; 
+	  dst[2] = (*src >> 0) & 0xff;
+	  if (pixbuf_n_channels == 4)
+	      dst[3] = (*src >> 24) & 0xff;
+	  dst += pixbuf_n_channels;
+	  src++;
+	}
+    }
+}
+
+static void
+_poppler_page_render_to_pixbuf (PopplerPage *page,
+				int src_x, int src_y,
+				int src_width, int src_height,
+				double scale,
+				int rotation,
+				GBool printing,
+				GdkPixbuf *pixbuf)
+{
+  cairo_t *cr;
+  cairo_surface_t *surface;
+
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					src_width, src_height);
+  cr = cairo_create (surface);
+  cairo_save (cr);
+  switch (rotation) {
+  case 90:
+	  cairo_translate (cr, src_x + src_width, -src_y);
+	  break;
+  case 180:
+	  cairo_translate (cr, src_x + src_width, src_y + src_height);
+	  break;
+  case 270:
+	  cairo_translate (cr, -src_x, src_y + src_height);
+	  break;
+  default:
+	  cairo_translate (cr, -src_x, -src_y);
+  }
+
+  if (scale != 1.0)
+	  cairo_scale (cr, scale, scale);
+
+  if (rotation != 0)
+	  cairo_rotate (cr, rotation * G_PI / 180.0);
+
+  if (printing)
+	  poppler_page_render_for_printing (page, cr);
+  else
+	  poppler_page_render (page, cr);
+  cairo_restore (cr);
+
+  cairo_set_operator (cr, CAIRO_OPERATOR_DEST_OVER);
+  cairo_set_source_rgb (cr, 1., 1., 1.);
+  cairo_paint (cr);
+
+  cairo_destroy (cr);
+
+  copy_cairo_surface_to_pixbuf (surface, pixbuf);
+  cairo_surface_destroy (surface);
+}
+
+/**
+ * poppler_page_render_to_pixbuf:
+ * @page: the page to render from
+ * @src_x: x coordinate of upper left corner  
+ * @src_y: y coordinate of upper left corner  
+ * @src_width: width of rectangle to render  
+ * @src_height: height of rectangle to render
+ * @scale: scale specified as pixels per point
+ * @rotation: rotate the document by the specified degree
+ * @pixbuf: pixbuf to render into
+ *
+ * First scale the document to match the specified pixels per point,
+ * then render the rectangle given by the upper left corner at
+ * (src_x, src_y) and src_width and src_height.
+ * This function is for rendering a page that will be displayed.
+ * If you want to render a page that will be printed use
+ * poppler_page_render_to_pixbuf_for_printing() instead
+ *
+ * Deprecated: 0.16
+ **/
+void
+poppler_page_render_to_pixbuf (PopplerPage *page,
+			       int src_x, int src_y,
+			       int src_width, int src_height,
+			       double scale,
+			       int rotation,
+			       GdkPixbuf *pixbuf)
+{
+  g_return_if_fail (POPPLER_IS_PAGE (page));
+  g_return_if_fail (scale > 0.0);
+  g_return_if_fail (pixbuf != NULL);
+
+  _poppler_page_render_to_pixbuf (page, src_x, src_y,
+				  src_width, src_height,
+				  scale, rotation,
+				  gFalse,
+				  pixbuf);
+}
+#endif
+
 static WebKitNavigationResponse navigation_requested(WebKitWebView *view, 
 		WebKitWebFrame *frame, 
 		WebKitNetworkRequest *netreq, 
@@ -61,9 +198,27 @@ static WebKitNavigationResponse navigation_requested(WebKitWebView *view,
 	return WEBKIT_NAVIGATION_RESPONSE_ACCEPT;
 }
 
-static void load_done(
-		WebKitWebFrame* frame, gboolean success,
-                gpointer data)
+static void load_status_changed(GObject *object, GParamSpec *pspec, gpointer data)
+{
+	WebKitWebView *view = WEBKIT_WEB_VIEW(object);
+	WebKitLoadStatus status = webkit_web_view_get_load_status(view);
+
+	if (status == WEBKIT_LOAD_FINISHED || status == WEBKIT_LOAD_FAILED)
+		load_done(webkit_web_view_get_main_frame(view), status);
+}
+
+static void load_resource (WebKitWebView     *web_view,
+                           WebKitWebFrame    *web_frame,
+                           WebKitWebResource *web_resource,
+                           gpointer           user_data)
+{
+	if(!gui_shown) {
+		printf(".");
+		fflush(stdout);
+	}
+}
+
+static void load_done(WebKitWebFrame *frame, WebKitLoadStatus status)
 {
 	gchar *format = g_strdup_printf(output_file_format,cur_url);
 	gchar *filename = g_strconcat(output_dir?output_dir:g_get_current_dir(),
@@ -74,8 +229,10 @@ static void load_done(
 	PopplerPage *page = NULL;
 	GdkPixbuf *pb;
 
-	g_free(format);
-	if (success && urls && urls[cur_url]) {
+	if(!gui_shown)
+		printf("\n");
+
+	if (status == WEBKIT_LOAD_FINISHED && urls && urls[cur_url]) {
 		GtkPrintOperation *operation = gtk_print_operation_new();
 		if (print_settings)
 			gtk_print_operation_set_print_settings(operation, print_settings);
@@ -159,7 +316,9 @@ static void load_done(
 			}
 		}
 		
-	}
+	} else if (status == WEBKIT_LOAD_FAILED)
+		printf("  Error\n");
+
 	g_free(filename);
 	cur_url++;
 	
@@ -172,8 +331,10 @@ static void load_done(
 			gtk_statusbar_push(GTK_STATUSBAR(statusbar), 
 				gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "status"),
 				text);
-			if (!gui_shown)
-				printf(" %s\n", text);
+			if (!gui_shown) {
+				printf(" %s", text);
+				fflush(stdout);
+			}
 			g_free(text);
 			webkit_web_view_open(WEBKIT_WEB_VIEW(webview), urls[cur_url]);
 			return;
@@ -373,8 +534,10 @@ static void do_print(GtkTextView *textview)
 	gtk_statusbar_push(GTK_STATUSBAR(statusbar), 
 		gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "status"),
 		text);
-	if (!gui_shown)
-		printf(" %s\n", text);
+	if (!gui_shown) {
+		printf(" %s", text);
+		fflush(stdout);
+	}
 	g_free(text);
 	webkit_web_view_open(WEBKIT_WEB_VIEW(webview), urls[cur_url]);
 }
@@ -476,13 +639,15 @@ int main(int argc, char *argv[])
 	GtkWidget *toolbar = NULL;
 	GtkWidget *label;
 	GtkWidget *scrolledwin;
-	WebKitWebFrame *frame;
 
 	bindtextdomain(PACKAGE, LOCALEDIR );
         bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
 
+#if !GLIB_CHECK_VERSION(2,32,0)
 	g_thread_init(NULL);
+#endif
+
 	gtk_init(&argc, &argv);
 	window =gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_widget_set_size_request(GTK_WIDGET(window), 800, 600);
@@ -494,6 +659,8 @@ int main(int argc, char *argv[])
 	vbox_pdf = gtk_vbox_new(FALSE, 6);
 	text = GTK_TEXT_VIEW(gtk_text_view_new());
 	toolbar = gtk_toolbar_new();
+	
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);
 	page_setup = gtk_page_setup_new();
 
 	label = gtk_label_new(_("URLs to print:"));
@@ -583,8 +750,10 @@ int main(int argc, char *argv[])
 					 G_CALLBACK(quit_cb),
 					 (gpointer *) NULL);
 
-	frame = webkit_web_view_get_main_frame(WEBKIT_WEB_VIEW(webview));
-	g_signal_connect(frame, "load-done", G_CALLBACK(load_done),NULL);
+	g_signal_connect(webview, "resource-load-finished", G_CALLBACK(load_resource),NULL);
+	g_signal_connect (webview, "notify::load-status",
+		G_CALLBACK (load_status_changed), NULL);
+
 
 	output_file_format = g_strdup("%04d.pdf");
 	if (argc < 2) {
